/*
  Moth Player: Delos Skin - Visualizations add-on
  (c) 2022 Moth. MIT License.

  Note: Script tailored for this demo (sizes, colors, selectors, functionality.)
  If you're looking for a cool, generic visualization library, check Wave.js:
  https://foobar404.dev/Wave.js/
*/

window.onload = function() {
  "use strict";
  // Canvas elements' width and height. 'w' affects the AudioContext arrays.
  const w = 256; // Easier life and sharp lines when powers of 2.
  const h = 64;

  // Canvas for the frequency data.
  const canvas1 = document.getElementById('visual1');
  let ctx1;
  if (canvas1) {
    canvas1.width  = w;
    canvas1.height = h;
    ctx1 = canvas1.getContext('2d');
  }

  // Canvas for the time domain data.
  const canvas2 = document.getElementById('visual2');
  let ctx2;
  if (canvas2) {
    canvas2.width  = w;
    canvas2.height = h;
    ctx2 = canvas2.getContext('2d');
  }

  const player = document.querySelector('.moth-player');
  const audio = document.getElementById('audio01');
  const bgc = window.getComputedStyle(player, null).backgroundColor;

  // Player controls start/stop the renderFrame loops.
  const ply = document.querySelector('.mp-play');
  const pau = document.querySelector('.mp-pause');
  const stp = document.querySelector('.mp-stop');
  const pll = document.querySelector('.mp-playlist');

  if (ply) ply.addEventListener('click', render);
  if (pau) pau.addEventListener('click', renderStop);
  if (stp) stp.addEventListener('click', renderStop);
  if (pll) pll.addEventListener('click', plClick);
  audio.addEventListener('play', end);

  let pause = true; // Needed to stop the renderFrame loops.

  // When we reach the last track, trigger event listener to
  // stop render unless repeat is on.
  function end() {
    const plc = pll.lastElementChild;
    let lastEnd = () => {
      const loop = document.querySelector('.mp-loop');
      if (loop) {
        loop.matches('.mp-looped') ? render() : renderStop();
      }
    };
    if (plc.matches('.mp-plist-current')) {
      audio.addEventListener('ended', lastEnd, {once: true});
    }
  }

  // Handle clicks on playlist items.
  function plClick (e) {
    let li = e.target.closest('li');
    if (!li) return;
    if (li.matches('.active')) {
      if (pause === true) render();
      else renderStop();
    }
    else {
      let clickedTrack = 0;
      while (li.previousElementSibling) {
        li = li.previousElementSibling;
        clickedTrack++;
      }
      const current = pll.querySelector('.active');
      if (current) current.classList.remove('active');
      pll.children[clickedTrack].classList.add('active');
      render();
    }
  }

  // Initial and paused states clean the canvas.
  function clear() {
    if (canvas1) {
      ctx1.fillStyle = bgc;
      ctx1.fillRect(0, 0, w, h);
    }
    if (canvas2) {
      ctx2.fillStyle = bgc;
      ctx2.fillRect(0, 0, w, h);
    }
  }

  clear(); // Do it on page load.

  // AudioContext interface. See footnote (1).
  const ct = new AudioContext();
  const src = ct.createMediaElementSource(audio);
  const an = ct.createAnalyser();
  src.connect(an);
  an.connect(ct.destination);
  an.minDecibels = -140;
  an.maxDecibels = 0;
  let x;
  let barHeight;

  // Frequency data graph
  function renderFrame1() {
    if (pause) return;
    an.fftSize = w / 2; // 128
    const bufferLength = an.frequencyBinCount; // 64 data values to represent
    const dataArray = new Uint8Array(bufferLength);
    const barWidth = w / bufferLength; // 4
    const f = 0.25; // Percentage factor of bar in barWidth: bar(1) + space between(3)
    x = 0; // Starting point in canvas (x-axis)
    requestAnimationFrame(renderFrame1);
    an.getByteFrequencyData(dataArray);
    ctx1.fillStyle = 'rgba(8, 9, 10, .1)';
    ctx1.fillRect(0, 0, w, h);

    // Number of bars in graph (== bufferLength)
    for (let i = 0; i < bufferLength; i++) {
      let value = dataArray[i];
      barHeight = h - h * (value / 256) - 1;
      ctx1.fillStyle = 'rgba(31, 175, 204, .7)';
      ctx1.fillRect(x, barHeight, barWidth * f, h);
      x += barWidth;
    }
  }

  // Time Domain data graph: 1 dataArray value for each x-axis canvas point.
  function renderFrame2() {
    if (pause) return;
    an.fftSize = w / 0.5; // 512
    const bufferLength = an.frequencyBinCount; // 256 data values to represent
    const dataArray = new Uint8Array(bufferLength);
    const barWidth = w / bufferLength; // 1
    requestAnimationFrame(renderFrame2);
    an.getByteTimeDomainData(dataArray);
    ctx2.fillStyle = 'rgba(8, 9, 10, .1)';
    ctx2.fillRect(0, 0, w, h);

    for (let i = 0; i < bufferLength; i++) {
      let value = dataArray[i];
      let percent = value / 256;
      let offset = h * (1.5 - percent * 2);
      ctx2.fillStyle = 'rgba(31, 175, 204, .7)';
      ctx2.fillRect(i * barWidth, offset, 1, 1);
    }
  }

  // Start renderFrame loops.
  function render() {
    pause = false;
    // Needed to comply with autoplay policy in Chrome(1).
    if (ct.state === 'suspended') ct.resume();
    if (canvas1) renderFrame1();
    if (canvas2) renderFrame2();
  }

  //Stop renderFrame loops.
  function renderStop() {
    pause = true;
    clear();
  }
};

// (1) https://developer.chrome.com/blog/autoplay/#webaudio
